/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Song.h,v 1.25 2003/02/22 13:07:48 comix Exp $
 *
 */

#ifndef SONG_H
#define SONG_H


#include <string>
using std::string;

#include <vector>
using std::vector;

#include "Object.h"
#include "Sample.h"
#include "LocalFileMng.h"


class Note;
class Instrument;
class Sequence;
class Pattern;
class Song;

//-----------------------------------------------------------------------------
/**
 * Sequence List
 */
class SequenceList : public Object {
	private:
		vector<Sequence*> list;
	public:
		virtual string getClassName() {	return "SequenceList";	}

		SequenceList();
		~SequenceList();

		void add(Sequence* newSequence);
		Sequence* get(unsigned int pos);
		unsigned int getSize();
	
		SequenceList* copy();
};




//-----------------------------------------------------------------------------
/**
 * Pattern List
 */
class PatternList : public Object {
	private:
		vector<Pattern*> list;
	
	public:
		virtual string getClassName() {	return "PatternList";	}

		PatternList();
		~PatternList();

		void add(Pattern* newPattern);
		Pattern* get(unsigned int pos);
		unsigned int getSize();
		void clear();

		void del(Pattern *pattern);
		void del(uint index);
};




//-----------------------------------------------------------------------------
/**
 * Instrument List
 */
class InstrumentList : public Object {
	private:
		vector<Instrument*> list;
	public:
		virtual string getClassName() {	return "InstrumentList";	}

		InstrumentList();
		~InstrumentList();

		void add(Instrument* newPattern);
		Instrument* get(unsigned int pos);
		unsigned int getSize();
		void set(Instrument* newInstrument, int pos);

};





//----------------------------------------------------------------------------
/**
 * A note...
 */
class Note : public Object{
	private:
		/** Note position inside the song */
		unsigned int position;

		/** Velocity (intensity) of the note */
		float velocity;

		/** Pan of the note (left volume) */
		float pan_L;

		/** Pan of the note (right volume) */
		float pan_R;

		Instrument* instrument;

	public:
		virtual string getClassName() {	return "Note";	}

		/** Constructor. */
		Note(unsigned int position,
			float velocity,
			float pan_L,
			float pan_R
		);

		/** Destructor */
		~Note();

		unsigned int getPosition(){	return position;	}
		void setPosition(unsigned int pos) {	this->position = pos;	}

		float getPan_L(){	return pan_L;	}
		void setPan_L( float val );

		float getPan_R(){	return pan_R;	}
		void setPan_R( float val );

		unsigned int samplePosition;

		void setInstrument(Instrument* instrument);
		Instrument* getInstrument();

		Note* copy();

		float getVelocity(){	return velocity;	}
		void setVelocity( float val );
};



//----------------------------------------------------------------------------
/**
 * Instrument class (sample + info)
 */
class Instrument : public Object{
	private:
		/** ID of the instrument */
		string id;

		/** Volume of the instrument */
		float volume;

		/** Instrument name */
		string name;

		/** author of the Instrument */
		string author;

		Sample *sample;

		bool muted;

		/** current peak value */
		float peak;

		/** Pan of the instrument (left volume) */
		float pan_L;

		/** Pan of the instrument (right volume) */
		float pan_R;

	public:
		virtual string getClassName() {	return "Instrument";	}

		/** Constructor. */
		Instrument( string id, string name, string author, int type, float volume );

		/** Destructor */
		~Instrument();

		/** Return the volume */
		float getVolume(){	return volume;	}
		void setVolume(float volume);

		void setId(string id) {	this->id = id;	}
		string getId(){	return id;	}

		string getName(){	return name;	}
		void setName(string name) {	this->name = name;	}

		string getAuthor(){	return author;	}

		void setSample(Sample *newSample){	sample = newSample;	}
		Sample* getSample(){	return sample;	}

		bool isMuted() {	return muted;	}
		void setMuted(bool muted) {	this->muted = muted;	}

		/** \todo implementation missing */
		static Instrument* load(string filename);

		/** \todo implementation missing */
		void save(string filename);

		void setPeak(float p) {	this->peak = p;	}
		float getPeak() {	return peak;	}


		float getPan_L(){	return pan_L;	}
		float getPan_R(){	return pan_R;	}
		void setPan_L( float val );
		void setPan_R( float val );

};





//----------------------------------------------------------------------------
/**
 * Sequence of notes
 */
class Sequence : public Object{
	private:
		string name;

	public:
		Note* noteList[MAX_NOTES];

		virtual string getClassName() {	return "Sequence";	}

		/** Constructor. */
		Sequence(string name);

		/** Destructor */
		~Sequence();

		string getName(){	return name;	}
		void setName(string newName){	name = newName;	}

		Sequence* copy();
};





//----------------------------------------------------------------------------
/**
 * Pattern (a container of sequences)
 */
class Pattern : public Object {
	private:
		string name;

		SequenceList *sequenceList;

	public:
		virtual string getClassName() {	return "Pattern";	}

		/** Constructor */
		Pattern(string name);

		/** Destructor */
		~Pattern();

		void setName(string name);
		string getName();

		SequenceList* getSequenceList(){	return sequenceList;	}
		void setSequenceList( SequenceList *vett ){	sequenceList = vett;	}
		
		static Pattern* getEmptyPattern();
		Pattern* copy();
};






//----------------------------------------------------------------------------
/**
 * Song class.
 */
class Song : public Object{
	private:
		/** ID of the song */
		string id;

		/** Beats per minute */
		unsigned int bpm;

		/** Resolution of the song */
		unsigned int resolution;

		/* volume of the song (0.0..1.0)*/
		float volume;

		/* Metronome volume */
		unsigned int metronomeVolume;

		/** song name */
		string name;

		/** author of the song */
		string author;

		/** Notes */
		string notes;

		/** List of patterns */
		PatternList *patternList;

		/** Sequence of patterns */
		PatternList *patternSequence;

		/** Track instruments */
		InstrumentList *instrumentList;

		/** Filename */
		string filename;

		/** is modified? */
		bool modified;

		bool loopEnabled;

	public:
		virtual string getClassName() {	return "Song";	}

		/** Constructor */
		Song(string id, string name, string author, unsigned int bpm, float volume);

		/** Destructor */
		~Song();

		string getId(){	return id;	}

		unsigned int getBpm(){	return bpm;	}
		void setBpm(unsigned int bpm);

		void setVolume(float volume) {	this->volume = volume;	}
		float getVolume() {	return volume;	}

		void setMetronomeVolume(unsigned int volume) {	this->metronomeVolume = volume;	}
		unsigned int getMetronomeVolume() {	return metronomeVolume;	}

		unsigned int getResolution(){	return resolution;	}


		string getName(){	return name;	}
		void setName(string name) {	this->name = name;	}

		void setAuthor(string author) {	this->author = author;	}
		string getAuthor(){	return author;	}

		PatternList* getPatternList(){	return patternList;	}
		void setPatternList( PatternList *vett ){	patternList = vett;	}

		PatternList* getPatternSequence(){	return patternSequence;	}
		void setPatternSequence( PatternList *vett ){	patternSequence = vett;	}

		static Song* load(string filename);
		void save(string filename);

		InstrumentList* getInstrumentList(){	return instrumentList;	}
		void setInstrumentList( InstrumentList *vett ){	instrumentList = vett;	}

		/** return the total number of ticks in the song */
		unsigned int estimateSongSize();

		static Song* getEmptySong();

		void setNotes(string newNotes) {	notes = newNotes;	}
		string getNotes() {	return notes;	}

		string getFilename() {	return filename;	}
		void setFilename(string filename) {	this->filename = filename;	}

		bool isModified() {	return modified;	}
		void setModified(bool status) {	this->modified = status;	}

		bool isLoopEnabled() {	return loopEnabled;	}
		void setLoopEnabled( bool enabled ) {	loopEnabled = enabled;	}
};


#endif


